//---------------------------------------------------------------------------
/*
GTST, Game Theory Server
Copyright (C) 2011 Richel Bilderbeek

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
//---------------------------------------------------------------------------
//From http://www.richelbilderbeek.nl/ProjectGtst.htm
//---------------------------------------------------------------------------
#ifndef SERVERSTATEVIEWRESULTSALL_H
#define SERVERSTATEVIEWRESULTSALL_H
//---------------------------------------------------------------------------
#include <map>
//---------------------------------------------------------------------------
#include "serverstate.h"
//---------------------------------------------------------------------------
///\brief
///The Server in the state when each Participant can view all
///other results.
struct ServerStateViewResultsAll : public ServerState, StateViewResultsAll
{
  ///Create this ServerState by passing the Server it is a State of
  ServerStateViewResultsAll(Server * const server);

  ///Check if this state can go to the next state.
  ///This method is implemented like this for possible future
  ///more complex state transitions
  bool CanGoToNextState() const;

  ///Get the groups their IPGG payoffs
  const std::map<const Group *,double>& GetGroupPayoffs() const { return m_group_payoffs; }

  ///Obtain all Participant's their states relevant for the current ServerState
  //const std::vector<std::string> GetParticipantStates() const;

  ///Go to the next state. Each state knows its next state.
  ///This method is implemented like this for possible future
  ///more complex state transitions
  void GoToNextState() const;

  ///ServerState dependent response to a timer
  void OnTimer();

  ///Obtain the duration of the state in seconds
  int GetStateDuration() const;

  ///Set the ServerStateViewResultsAll its ParametersViewResultsAll
  void SetParameters(const ParametersViewResultsAll* const parameters);

  ///Get the Participant-dependent results
  const std::vector<std::string> GetResults(
    const Participant * const participant) const;

  ///Start or restart the state, for example by resetting the timer
  void Start();

  ///Represent this ServerState as a std::string
  const std::string ToStr() const { return this->StateViewResultsAll::ToStr(); }

  private:
  ///Only allow a Boost smart pointer to delete ServerStateViewResultsAll
  //to prevent the following trouble,
  //cited from http://www.boost.org/libs/utility/checked_delete.html:
  //The C++ Standard allows, in 5.3.5/5, pointers to incomplete
  //class types to be deleted with a delete-expression.
  //When the class has a non-trivial destructor, or a class-specific operator
  //delete, the behavior is undefined. Some compilers issue a warning when an
  //incomplete type is deleted, but unfortunately, not all do, and programmers
  //sometimes ignore or disable warnings.
  ~ServerStateViewResultsAll() {}
  ///Only let smart pointers delete ServerStateViewResultsAll
  //Template syntax from Herb Sutter. Exceptional C++ style. 2005. ISBN: 0-201-76042-8. Item 8: 'Befriending templates'.
  friend void boost::checked_delete<>(ServerStateViewResultsAll*);


  ///The ServerStateViewResultsAll its read-only ParametersViewResultsAll
  const ParametersViewResultsAll * m_parameters;

  ///The number of repeats left
  int m_repeats_left;

  ///\brief
  ///The group its payoff in the last IPGG cycle
  ///
  ///int = key = group_index
  ///std::pair = value:
  ///- double: average payoff
  std::map<const Group *,double> m_group_payoffs;
};
//---------------------------------------------------------------------------
#endif // ServerSTATEVIEWRESULTSALL_H
